/* ***** BEGIN LICENSE BLOCK *****
 
 
 Copyright 2012 Adam Skillen (a_skil(at)encs.concordia.ca), Mohammad Mannan (mmannan(at)encs.concordia.ca)
 
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.  Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.  Neither the name of Carleton
 * University nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written
 * permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 * ***** END LICENSE BLOCK ***** */

  /**
  * 09-MAY-12
  * 	TODO:  1
  * 	FIXME: 0
  * 	NOTE:  1
  */
 
 Components.utils.import("resource://myphrase/common.js");
 Components.utils.import("resource://myphrase/commonWords.js");
 Components.utils.import("resource://myphrase/sha1.js");
 Components.utils.import("resource://myphrase/pbkdf2.js");
 Components.utils.import("resource://myphrase/gen_pwdhash.js");
 
/**
 * MyphraseCCSLOptions namespace.
 */
if ("undefined" == typeof(MyphraseCCSLGetMaster)) {
  var MyphraseCCSLGetMaster = {};
};


MyphraseCCSLGetMaster.BrowserOverlay = {
  prefs: null,
  promptService: null,
  words: null,
  gCurInput: "",
  
  onLoad: function() {
	this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
	  .getService(Components.interfaces.nsIPrefService)
	  .getBranch("extensions.myphrase.");

	this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                              .getService(Components.interfaces.nsIPromptService);

	// Tell the user if their preferences are not set
	var pathToDictionary = MyphraseCCSLGetMaster.BrowserOverlay.prefs.getCharPref("pathToDict");
	if (pathToDictionary == "") {
		promptService.alert(null, "No Dictionary File", "Please set the path to your dictionary file in the Myphrase preferences" +
		"\nThat will allow Myphrase to suggest words based on the letters you type");
		
  }
	
	this.init();

  },
  
  init: function() {
  
	// collect domain
	var domain = Components.classes["@mozilla.org/network/effective-tld-service;1"]
              .getService(Components.interfaces.nsIEffectiveTLDService)
              .getBaseDomainFromHost(window.arguments[0].ownerDocument.domain, 0);
		
	// collect domain specific preferences
	var prefService = Components.classes["@mozilla.org/content-pref/service;1"]
              .getService(Components.interfaces.nsIContentPrefService);
    
    if (prefService.hasPref(domain, "myphrase.iteration")) {
    	var iterate = prefService.getPref(domain, "myphrase.iteration");
		var specialChars = prefService.getPref(domain, "myphrase.specialChars");
		var passLength = prefService.getPref(domain, "myphrase.passLength");
		
    // set defaults if prefs don't exist yet
	} else {
    	prefService.setPref(domain, "myphrase.iteration", 0);
		prefService.setPref(domain, "myphrase.specialChars", false);
		prefService.setPref(domain, "myphrase.passLength", 12);
    	var iterate = prefService.getPref(domain, "myphrase.iteration");
		var specialChars = prefService.getPref(domain, "myphrase.specialChars");
		var passLength = prefService.getPref(domain, "myphrase.passLength");
    }
    
    // update local pref fields
    document.getElementById("iterate").value = iterate;
	document.getElementById("specialChars").checked = specialChars;
	document.getElementById("passLength").value = passLength;
    
  
	// load dictionary into an nsIFile from prefs string
	var dictFilePath = MyphraseCCSLGetMaster.BrowserOverlay.prefs.getCharPref("pathToDict");
	var data = MyphraseCCSL.Common.GetFileContent("file://"+dictFilePath);
	
	// get pwdLength from prefs
	var pwdLength = MyphraseCCSLGetMaster.BrowserOverlay.prefs.getIntPref("masterLengthPref");

	// load dictionary words into an array (account for CR+LF)
	data = data.replace(/\s{2,}/g," ");
	this.words = data.split(/\s/);
	
	// add common words to user's words
	var freqWords = [];
	freqWords = freqWords.concat(MyphraseCCSL.words.CC, 
		MyphraseCCSL.words.DT, MyphraseCCSL.words.EX, MyphraseCCSL.words.IN, 
		MyphraseCCSL.words.MD, MyphraseCCSL.words.PRP$, MyphraseCCSL.words.PRP, 
		MyphraseCCSL.words.RP, MyphraseCCSL.words.TO, MyphraseCCSL.words.UH, 
		MyphraseCCSL.words.WP);
	this.words = this.words.concat(freqWords);
	
	// sort words lexicographically
	this.words.sort();
	
	document.getElementById("passphraseField").focus();
  },
  
  showInput: function(cBox) {
	if (document.getElementById("shadow").checked) {
		document.getElementById("passphraseField").type = "";
	} else {
		document.getElementById("passphraseField").type = "password";
	}
		
  },
  
  onInputPwd: function(input) {
	var inputToken = input.split(/\s/);
	var str = inputToken[inputToken.length-1];
		
	// find length of array
	var numWords = this.words.length;
	
    // if the user didn't use auto-complete, (she typed a space) update the current input string
	if (input.charAt(input.length - 1) == " ")
		this.gCurInput = input;

	if (inputToken.length == 1)
		this.gCurInput = "";

	var ret = "<html:ul>";
	
	var regex = new RegExp("^" + str, "i");
	for (var i = 0; i < numWords; i++) {
		var sugWord = this.words[i].match(regex);
			if (sugWord) {
				ret += '<html:li onclick="MyphraseCCSLGetMaster.BrowserOverlay.setCurrentWord(\'' + this.words[i] + '\');"';
				ret += ' onmouseout="this.style.backgroundColor=\'transparent\'"  onmouseover="this.style.cursor=\'pointer\'; this.style.backgroundColor=\'lightblue\';">' + this.words[i] + "</html:li>";
			}
		}
		ret += "</html:ul>";
		
		if (str.length==0) {
			document.getElementById("livesearch").innerHTML="";
			return;
		} else {
			document.getElementById("livesearch").innerHTML=ret;
		}
},


setCurrentWord: function(word) {
  this.gCurInput += (word + " ");
  document.getElementById("passphraseField").value = this.gCurInput;
  document.getElementById("livesearch").innerHTML="";
  document.getElementById("livesearch").style.border="0px";
},

  GenSitePwd: function(event) {
	var obj  = window.arguments[0];

	if(obj && obj.type.toLowerCase() == "password") 
	{
		// collect domain
		var eTLDService = Components.classes["@mozilla.org/network/effective-tld-service;1"]
                  .getService(Components.interfaces.nsIEffectiveTLDService);
	
		var domain = eTLDService.getBaseDomainFromHost(obj.ownerDocument.domain, 0);
		
		// update domain specific preferences
		var prefService = Components.classes["@mozilla.org/content-pref/service;1"]
		          .getService(Components.interfaces.nsIContentPrefService);
		          
		var iterate = document.getElementById("iterate").value;
		var specialChars = document.getElementById("specialChars").checked;
		var passLength = document.getElementById("passLength").value;
		prefService.setPref(domain, "myphrase.iteration", iterate);
		prefService.setPref(domain, "myphrase.specialChars", specialChars);
		prefService.setPref(domain, "myphrase.passLength", passLength);
	
	
		
		// backward compatibility for no update counter
		if (iterate < 0) {
			iterate = "";
		}
		
		// get phassphrase
		var masterPass = document.getElementById("passphraseField").value;
		
		// trim trailing whitespace
		masterPass = masterPass.replace(/\s*/g, "");
		if(masterPass == "") {
			return;
		}

		// TODO: Respect the user's hash count
		// var hashCount = this.prefs.getIntPref("hashCountPref");
		var hashCount = 32768;
		
		//XXX 
		//alert(masterPass + " | " + domain + " | " + iterate);
		
		//XXX
		var time1 = new Date().getTime();
		
		// make H(passphrase|domain|iteration)
		 try{
		   var b64hash = MyphraseCCSL.PBKDF2_impl.pbkdf2_b64(masterPass, domain + iterate, hashCount);
		 } catch(e)
		 {
		   alert("Myphrase: Error generating hash from object data.");
		   return;	
		 }
		 
		 //XXX
		 //var time2 = new Date().getTime();
		 //var time = time2 - time1;
		 //alert("time: " + time);
		 
		//XXX  
		//alert(b64hash + " | " + specialChars + " | " + passLength);		 
		 
		// make base64(H)
		 var sitePwd = MyphraseCCSL.PwdHashImpl.SPH_HashedPassword(b64hash, specialChars, passLength);

		// insert encoded hash value (aka site password) into the current field
		obj.value = sitePwd;

		
		// NOTE: Should we allow the user to copy the password to the clipboard?
		
		// XXX
		//alert(sitePwd);
	}
  },

};

window.addEventListener("load", function(e) { MyphraseCCSLGetMaster.BrowserOverlay.onLoad(e); }, false);
