/*
 * JavaScript implementation of Password-Based Key Derivation Function 2
 * (PBKDF2) as defined in RFC 2898.
 * Version 1.3 
 * Copyright (c) 2007, 2008, 2009, 2010, 2011, 2012 Parvez Anandam
 * parvez.anandam@cern.ch
 * http://anandam.name/pbkdf2
 *
 * Distributed under the BSD license
 *
 * Uses Paul Johnston's excellent SHA-1 JavaScript library sha1.js:
 * http://pajhome.org.uk/crypt/md5/sha1.html
 * (uses the str2binb() function from that libary)
 *
 * Thanks to Felix Gartsman for pointing out a bug in version 1.0
 * Thanks to Thijs Van der Schaeghe for pointing out a bug in version 1.1 
 * Thanks to Richard Gautier for asking to clarify dependencies in version 1.2
 */

/**
 * Modified by Adam Skillen (a_skil _at_ encs.concordia.ca)
 * Changed into a js class: 10-Oct-12
 */


var EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://myphrase/common.js");
Components.utils.import("resource://myphrase/sha1.js");



MyphraseCCSL.PBKDF2_impl = {
    pbkdf2_raw: function(password, salt, num_iterations) {
        return this._PBKDF2(password, salt, num_iterations, 20);
    },
    
    pbkdf2_hex: function(password, salt, num_iterations) {
       return MyphraseCCSL.Sha1HashImpl.rstr2hex(this._PBKDF2(password, salt, num_iterations, 20));
    },
    
    pbkdf2_b64: function(password, salt, num_iterations) {
        return MyphraseCCSL.Sha1HashImpl.rstr2b64(this._PBKDF2(password, salt, num_iterations, 20));
    },

    _PBKDF2: function(password, salt, num_iterations, num_bytes){

	    var m_buffer = [];  //result of the iterations
	    var m_hash = [];    // The hash cache	    
	    var m_key = "";     // The derived key

	    // HMAC-SHA1 output is 20 bytes.
	    var m_hash_length = 20;

	    // Number of hash-sized blocks in the derived key (called 'l' in RFC2898)
	    var m_total_blocks = Math.ceil(num_bytes/m_hash_length);

        // for each block
        for (current_block = 1; current_block <= m_total_blocks; current_block++) {
            
            var salt_block = salt + String.fromCharCode(current_block >> 24 & 0xF) +
						    String.fromCharCode(current_block >> 16 & 0xF) +
						    String.fromCharCode(current_block >>  8 & 0xF) +
						    String.fromCharCode(current_block       & 0xF);
            
            
            m_buffer = m_hash = MyphraseCCSL.Sha1HashImpl.rstr_hmac_sha1(password, salt_block);

            // for each iteration
            for (var i = 1; i < num_iterations; i++) {
            
                var t_buffer = "";

                m_hash = MyphraseCCSL.Sha1HashImpl.rstr_hmac_sha1(password, m_hash);

                for(var j=0; j<m_hash.length; j++)
                    t_buffer += String.fromCharCode(m_buffer.charCodeAt(j) ^ m_hash.charCodeAt(j));             
                m_buffer = t_buffer;
            }
            m_key += m_buffer.substr(0, (num_bytes - (m_total_blocks - 1) * m_hash_length) * 2 );
        }
        return m_key;
    },
}
